@@ -1,4 +1,4 @@
-/*	$NetBSD: iommu.c,v 1.108 2014/08/24 19:09:43 palle Exp $	*/
+/*	$NetBSD: iommu.c,v 1.110 2015/09/06 23:47:09 nakayama Exp $	*/
 
 /*
  * Copyright (c) 1999, 2000 Matthew R. Green
@@ -59,7 +59,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: iommu.c,v 1.108 2014/08/24 19:09:43 palle Exp $");
+__KERNEL_RCSID(0, "$NetBSD: iommu.c,v 1.110 2015/09/06 23:47:09 nakayama Exp $");
 
 #include "opt_ddb.h"
 
@@ -78,13 +78,14 @@ __KERNEL_RCSID(0, "$NetBSD: iommu.c,v 1.108 2014/08/24 19:09:43 palle Exp $");
 
 #include <machine/autoconf.h>
 #include <machine/cpu.h>
+#include <machine/hypervisor.h>
 
 #ifdef DEBUG
 #define IDB_BUSDMA	0x1
 #define IDB_IOMMU	0x2
 #define IDB_INFO	0x4
 #define	IDB_SYNC	0x8
-int iommudebug = 0x00;
+int iommudebug = 0x0;
 #define DPRINTF(l, s)   do { if (iommudebug & l) printf s; } while (0)
 #define IOTTE_DEBUG(n)	(n)
 #else
@@ -215,7 +216,7 @@ iommu_init(char *name, struct iommu_state *is, int tsbsize, uint32_t iovabase)
 	    is->is_dvmabase, is->is_dvmaend,
 	    0, 0, EX_NOWAIT);
 	if (!is->is_dvmamap)
-	  panic("iommu_init: extent_create() failed");
+		panic("iommu_init: extent_create() failed");
 	  
 	mutex_init(&is->is_lock, MUTEX_DEFAULT, IPL_HIGH);
 
@@ -231,8 +232,7 @@ iommu_init(char *name, struct iommu_state *is, int tsbsize, uint32_t iovabase)
 	/*
 	 * now actually start up the IOMMU
 	 */
-	if (!CPU_ISSUN4V)
-		iommu_reset(is);
+	iommu_reset(is);
 }
 
 /*
@@ -246,6 +246,9 @@ iommu_reset(struct iommu_state *is)
 	int i;
 	struct strbuf_ctl *sb;
 
+	if (CPU_ISSUN4V)
+		return;
+	
 	IOMMUREG_WRITE(is, iommu_tsb, is->is_ptsb);
 
 	/* Enable IOMMU in diagnostic mode */
@@ -359,7 +362,7 @@ iommu_enter_sun4v(struct strbuf_ctl *sb, vaddr_t va, int64_t pa, int flags)
 	err = hv_pci_iommu_map(is->is_devhandle, tsbid, 1, attr,
 	    addr, &nmapped);
 	if (err != H_EOK || nmapped != 1)
-		panic("hv_pci_iommu_map: err=%d, nmapped=%lu", err, nmapped);
+		panic("hv_pci_iommu_map: err=%d, nmapped=%lu", err, (long unsigned int)nmapped);
 }
 
 /*
