@@ -76,9 +76,14 @@
 #define IDB_BREAK	0x40
 extern int iommudebug;
 #define DPRINTF(l, s)   do { if (iommudebug & l) printf s; } while (0)
+#define IOTTE_DEBUG(n)	(n)
 #else
 #define DPRINTF(l, s)
+#define IOTTE_DEBUG(n)	0
 #endif
+
+static	int viommu_strbuf_flush_done(struct strbuf_ctl *);
+
 #if 0
 FiXME
 void viommu_enter(struct iommu_state *, struct strbuf_ctl *, bus_addr_t,
@@ -134,9 +139,11 @@ viommu_init(char *name, struct iommu_state *is, int tsbsize,
 	is->is_dvmamap = extent_create(name,
 	    is->is_dvmabase, (u_long)is->is_dvmaend + 1,
 	    NULL, 0, EX_NOCOALESCE);
+	if (!is->is_dvmamap)
+	  panic("viommu_init: extent_create() failed");
 	mutex_init(&is->is_lock, MUTEX_DEFAULT, IPL_HIGH);
 
-	printf("\n");
+	printf("\nviommu_init() done %p\n", (void*)is->is_dvmamap);
 }
 
 #if 0
@@ -268,23 +275,293 @@ viommu_dvmamap_destroy(bus_dma_tag_t t, bus_dmamap_t map)
 }
 
 
-/*
- * Load a contiguous kva buffer into a dmamap.  The physical pages are
- * not assumed to be contiguous.  Two passes are made through the buffer
- * and both call pmap_extract() for the same va->pa translations.  It
- * is possible to run out of pa->dvma mappings; the code should be smart
- * enough to resize the iomap (when the "flags" permit allocation).  It
- * is trivial to compute the number of entries required (round the length
- * up to the page size and then divide by the page size)...
- */
+static int
+viommu_strbuf_flush_done(struct strbuf_ctl *sb)
+{
+	struct iommu_state *is = sb->sb_is;
+	struct timeval cur, flushtimeout;
+
+#define BUMPTIME(t, usec) { \
+	register volatile struct timeval *tp = (t); \
+	register long us; \
+ \
+	tp->tv_usec = us = tp->tv_usec + (usec); \
+	if (us >= 1000000) { \
+		tp->tv_usec = us - 1000000; \
+		tp->tv_sec++; \
+	} \
+}
+
+	if (!sb->sb_flush)
+		return (0);
+
+	/*
+	 * Streaming buffer flushes:
+	 *
+	 *   1 Tell strbuf to flush by storing va to strbuf_pgflush.  If
+	 *     we're not on a cache line boundary (64-bits):
+	 *   2 Store 0 in flag
+	 *   3 Store pointer to flag in flushsync
+	 *   4 wait till flushsync becomes 0x1
+	 *
+	 * If it takes more than .5 sec, something
+	 * went wrong.
+	 */
+
+	*sb->sb_flush = 0;
+	bus_space_write_8(is->is_bustag, sb->sb_sb,
+		STRBUFREG(strbuf_flushsync), sb->sb_flushpa);
+
+	microtime(&flushtimeout);
+	cur = flushtimeout;
+	BUMPTIME(&flushtimeout, 500000); /* 1/2 sec */
+
+	DPRINTF(IDB_IOMMU, ("%s: flush = %lx at va = %lx pa = %lx now="
+		"%"PRIx64":%"PRIx32" until = %"PRIx64":%"PRIx32"\n", __func__,
+		(long)*sb->sb_flush, (long)sb->sb_flush, (long)sb->sb_flushpa,
+		cur.tv_sec, cur.tv_usec,
+		flushtimeout.tv_sec, flushtimeout.tv_usec));
+
+	/* Bypass non-coherent D$ */
+	while ((!ldxa(sb->sb_flushpa, ASI_PHYS_CACHED)) &&
+	       timercmp(&cur, &flushtimeout, <=))
+		microtime(&cur);
+
+#ifdef DIAGNOSTIC
+	if (!ldxa(sb->sb_flushpa, ASI_PHYS_CACHED)) {
+		printf("%s: flush timeout %p, at %p\n", __func__,
+			(void *)(u_long)*sb->sb_flush,
+			(void *)(u_long)sb->sb_flushpa); /* panic? */
+#ifdef DDB
+		Debugger();
+#endif
+	}
+#endif
+	DPRINTF(IDB_IOMMU, ("%s: flushed\n", __func__));
+	return (*sb->sb_flush);
+}
+
+
 int
 viommu_dvmamap_load(bus_dma_tag_t t, bus_dmamap_t map,
     void *buf, bus_size_t buflen, struct proc *p, int flags)
 {
-#if 1
-  panic("FIXME viommu_dvmamap_load()");
-#else  
   printf("viommu_dvmamap_load()\n");
+#if 1
+	struct strbuf_ctl *sb = (struct strbuf_ctl *)map->_dm_cookie;
+	printf("%s:%d sb %p\n", __FILE__, __LINE__, (void*)sb);
+	struct iommu_state *is = sb->sb_is;
+	printf("%s:%d is %p\n", __FILE__, __LINE__, (void*)is);
+	int err, needsflush;
+	bus_size_t sgsize;
+	paddr_t curaddr;
+	u_long dvmaddr, sgstart, sgend, bmask;
+	bus_size_t align, boundary, len;
+	vaddr_t vaddr = (vaddr_t)buf;
+	int seg;
+	struct pmap *pmap;
+	int slot;
+
+	if (map->dm_nsegs) {
+		/* Already in use?? */
+#ifdef DIAGNOSTIC
+		printf("iommu_dvmamap_load: map still in use\n");
+#endif
+		bus_dmamap_unload(t, map);
+	}
+
+	/*
+	 * Make sure that on error condition we return "no valid mappings".
+	 */
+	map->dm_nsegs = 0;
+	KASSERT(map->dm_maxsegsz <= map->_dm_maxmaxsegsz);
+
+	if (buflen > map->_dm_size) {
+		DPRINTF(IDB_BUSDMA,
+		    ("iommu_dvmamap_load(): error %d > %d -- "
+		     "map size exceeded!\n", (int)buflen, (int)map->_dm_size));
+		return (EINVAL);
+	}
+
+	sgsize = round_page(buflen + ((int)vaddr & PGOFSET));
+
+	/*
+	 * A boundary presented to bus_dmamem_alloc() takes precedence
+	 * over boundary in the map.
+	 */
+	if ((boundary = (map->dm_segs[0]._ds_boundary)) == 0)
+		boundary = map->_dm_boundary;
+	align = max(map->dm_segs[0]._ds_align, PAGE_SIZE);
+
+	/*
+	 * If our segment size is larger than the boundary we need to
+	 * split the transfer up int little pieces ourselves.
+	 */
+	printf("%s:%d %p\n", __FILE__, __LINE__, (void*)is->is_dvmamap);
+	KASSERT(is->is_dvmamap);
+	printf("%s:%d\n", __FILE__, __LINE__);
+	mutex_enter(&is->is_lock);
+printf("%s:%d\n", __FILE__, __LINE__);
+	err = extent_alloc(is->is_dvmamap, sgsize, align,
+	    (sgsize > boundary) ? 0 : boundary,
+	    EX_NOWAIT|EX_BOUNDZERO, &dvmaddr);
+printf("%s:%d\n", __FILE__, __LINE__);
+	mutex_exit(&is->is_lock);
+printf("%s:%d\n", __FILE__, __LINE__);
+
+#ifdef DEBUG
+	if (err || (dvmaddr == (u_long)-1)) {
+		printf("iommu_dvmamap_load(): extent_alloc(%d, %x) failed!\n",
+		    (int)sgsize, flags);
+#ifdef DDB
+		Debugger();
+#endif
+	}
+#endif
+printf("%s:%d\n", __FILE__, __LINE__);
+	if (err != 0)
+		return (err);
+
+printf("%s:%d\n", __FILE__, __LINE__);
+	if (dvmaddr == (u_long)-1)
+		return (ENOMEM);
+
+printf("%s:%d\n", __FILE__, __LINE__);
+	/* Set the active DVMA map */
+	map->_dm_dvmastart = dvmaddr;
+	map->_dm_dvmasize = sgsize;
+
+	/*
+	 * Now split the DVMA range into segments, not crossing
+	 * the boundary.
+	 */
+	seg = 0;
+	sgstart = dvmaddr + (vaddr & PGOFSET);
+	sgend = sgstart + buflen - 1;
+	map->dm_segs[seg].ds_addr = sgstart;
+	DPRINTF(IDB_INFO, ("iommu_dvmamap_load: boundary %lx boundary - 1 %lx "
+	    "~(boundary - 1) %lx\n", (long)boundary, (long)(boundary - 1),
+	    (long)~(boundary - 1)));
+	bmask = ~(boundary - 1);
+printf("%s:%d\n", __FILE__, __LINE__);
+	while ((sgstart & bmask) != (sgend & bmask) ||
+	       sgend - sgstart + 1 > map->dm_maxsegsz) {
+printf("%s:%d\n", __FILE__, __LINE__);
+		/* Oops. We crossed a boundary or large seg. Split the xfer. */
+		len = map->dm_maxsegsz;
+		if ((sgstart & bmask) != (sgend & bmask))
+			len = min(len, boundary - (sgstart & (boundary - 1)));
+		map->dm_segs[seg].ds_len = len;
+		DPRINTF(IDB_INFO, ("iommu_dvmamap_load: "
+		    "seg %d start %lx size %lx\n", seg,
+		    (long)map->dm_segs[seg].ds_addr,
+		    (long)map->dm_segs[seg].ds_len));
+		if (++seg >= map->_dm_segcnt) {
+			/* Too many segments.  Fail the operation. */
+			DPRINTF(IDB_INFO, ("iommu_dvmamap_load: "
+			    "too many segments %d\n", seg));
+			mutex_enter(&is->is_lock);
+			err = extent_free(is->is_dvmamap,
+			    dvmaddr, sgsize, EX_NOWAIT);
+			map->_dm_dvmastart = 0;
+			map->_dm_dvmasize = 0;
+			mutex_exit(&is->is_lock);
+			if (err != 0)
+				printf("warning: %s: %" PRId64
+				    " of DVMA space lost\n", __func__, sgsize);
+			return (EFBIG);
+		}
+		sgstart += len;
+		map->dm_segs[seg].ds_addr = sgstart;
+	}
+	map->dm_segs[seg].ds_len = sgend - sgstart + 1;
+	DPRINTF(IDB_INFO, ("iommu_dvmamap_load: "
+	    "seg %d start %lx size %lx\n", seg,
+	    (long)map->dm_segs[seg].ds_addr, (long)map->dm_segs[seg].ds_len));
+	map->dm_nsegs = seg + 1;
+	map->dm_mapsize = buflen;
+
+	if (p != NULL)
+		pmap = p->p_vmspace->vm_map.pmap;
+	else
+		pmap = pmap_kernel();
+
+	needsflush = 0;
+	for (; buflen > 0; ) {
+printf("%s:%d\n", __FILE__, __LINE__);
+
+		/*
+		 * Get the physical address for this page.
+		 */
+		if (pmap_extract(pmap, (vaddr_t)vaddr, &curaddr) == FALSE) {
+#ifdef DIAGNOSTIC
+			printf("iommu_dvmamap_load: pmap_extract failed %lx\n", vaddr);
+#endif
+			bus_dmamap_unload(t, map);
+			return (-1);
+		}
+
+printf("%s:%d\n", __FILE__, __LINE__);
+		/*
+		 * Compute the segment size, and adjust counts.
+		 */
+		sgsize = PAGE_SIZE - ((u_long)vaddr & PGOFSET);
+		if (buflen < sgsize)
+			sgsize = buflen;
+
+		DPRINTF(IDB_BUSDMA,
+		    ("iommu_dvmamap_load: map %p loading va %p "
+		    "dva %lx at pa %lx\n",
+		    map, (void *)vaddr, (long)dvmaddr,
+		    (long)trunc_page(curaddr)));
+printf("%s:%d\n", __FILE__, __LINE__);
+		iommu_enter(sb, trunc_page(dvmaddr), trunc_page(curaddr),
+		    flags | IOTTE_DEBUG(0x4000));
+printf("%s:%d\n", __FILE__, __LINE__);
+		needsflush = 1;
+
+		vaddr += sgsize;
+		buflen -= sgsize;
+
+		/* Flush cache if necessary. */
+		slot = IOTSBSLOT(trunc_page(dvmaddr), is->is_tsbsize);
+printf("%s:%d\n", __FILE__, __LINE__);
+		if ((is->is_flags & IOMMU_FLUSH_CACHE) &&
+		    (buflen <= 0 || (slot % 8) == 7)) {
+printf("%s:%d\n", __FILE__, __LINE__);
+			IOMMUREG_WRITE(is, iommu_cache_flush,
+			    is->is_ptsb + slot * 8);
+printf("%s:%d\n", __FILE__, __LINE__);
+		}
+		dvmaddr += PAGE_SIZE;
+	}
+printf("%s:%d\n", __FILE__, __LINE__);
+	if (needsflush) 
+		viommu_strbuf_flush_done(sb);
+printf("%s:%d\n", __FILE__, __LINE__);
+#ifdef DIAGNOSTIC
+	for (seg = 0; seg < map->dm_nsegs; seg++) {
+		if (map->dm_segs[seg].ds_addr < is->is_dvmabase ||
+			map->dm_segs[seg].ds_addr > is->is_dvmaend) {
+			printf("seg %d dvmaddr %lx out of range %x - %x\n",
+			    seg, (long)map->dm_segs[seg].ds_addr,
+			    is->is_dvmabase, is->is_dvmaend);
+#ifdef DDB
+			Debugger();
+#endif
+		}
+	}
+#endif
+printf("%s:%d\n", __FILE__, __LINE__);
+	return (0);
+
+
+
+
+	
+#else  
+  OpenBSD version...
+
 	int err = 0;
 	bus_size_t sgsize;
 	u_long dvmaddr, sgstart, sgend;
@@ -449,8 +726,8 @@ viommu_dvmamap_load(bus_dma_tag_t t, bus_dmamap_t map,
 		viommu_dvmamap_unload(t, map);
 
 	return (err);
-#endif
 	return 0;
+#endif
 }
 
 
@@ -921,7 +1198,6 @@ viommu_dvmamem_alloc(bus_dma_tag_t t, bus_size_t size,
     bus_size_t alignment, bus_size_t boundary, bus_dma_segment_t *segs,
     int nsegs, int *rsegs, int flags)
 {
-#if 1
 	DPRINTF(IDB_BUSDMA, ("viommu_dvmamem_alloc: sz %llx align %llx "
 	    "bound %llx segp %p flags %d\n", (unsigned long long)size,
 	    (unsigned long long)alignment, (unsigned long long)boundary,
@@ -929,11 +1205,6 @@ viommu_dvmamem_alloc(bus_dma_tag_t t, bus_size_t size,
 	
 	return (bus_dmamem_alloc(t->_parent, size, alignment, boundary,
 	    segs, nsegs, rsegs, flags | BUS_DMA_DVMA));
-#else
-	panic("FIXME viommu_dvmamem_alloc()");
-	return 0;
-#endif
-	
 }
 
 
@@ -958,10 +1229,6 @@ int
 viommu_dvmamem_map(bus_dma_tag_t t, bus_dma_segment_t *segs, int nsegs,
 	size_t size, void **kvap, int flags)
 {
-#if 0
-  panic("FIXME viommu_dvmamem_map()");
-#else
-  printf("viommu_dvmamem_map()");
 	struct vm_page *pg;
 	vaddr_t va;
 	bus_addr_t addr;
@@ -1011,7 +1278,6 @@ viommu_dvmamem_map(bus_dma_tag_t t, bus_dma_segment_t *segs, int nsegs,
 		size -= PAGE_SIZE;
 	}
 	pmap_update(pmap_kernel());
-#endif	
 	return (0);
 }
 
