@@ -1,4 +1,4 @@
-/*	$NetBSD: vpci.c,v 1.1 2015/02/12 04:48:37 palle Exp $	*/
+/*	$NetBSD: vpci.c,v 1.5 2015/09/09 03:47:58 palle Exp $	*/
 /*
  * Copyright (c) 2015 Palle Lyckegaard
  * All rights reserved.
@@ -30,7 +30,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: vpci.c,v 1.1 2015/02/12 04:48:37 palle Exp $");
+__KERNEL_RCSID(0, "$NetBSD: vpci.c,v 1.5 2015/09/09 03:47:58 palle Exp $");
 
 #include <sys/param.h>
 #include <sys/device.h>
@@ -53,7 +53,7 @@ __KERNEL_RCSID(0, "$NetBSD: vpci.c,v 1.1 2015/02/12 04:48:37 palle Exp $");
 #include <sparc64/dev/iommuvar.h>
 #include <sparc64/dev/vpcivar.h>
 
-#include <sparc64/hypervisor.h>
+#include <machine/hypervisor.h>
 
 #ifdef DEBUG
 #define VDB_PROM             0x01
@@ -233,6 +233,8 @@ FiXME
 	for (int range = 0; range < nranges; range++)
 		DPRINTF(VDB_PROM, ("\nvpci_attach: bus-range %d %08x", range, busranges[range]));
 
+ 	aprint_normal(": bus %d to %d", busranges[0], busranges[1]);
+	
 	vpci_init_iommu(sc, pbm);
 
 	pbm->vp_memt = vpci_alloc_mem_tag(pbm);
@@ -303,7 +305,10 @@ vpci_init_iommu(struct vpci_softc *sc, struct vpci_pbm *pbm)
 		    iobase, iolen, tsbsize));
 		free(vdma, M_DEVBUF);
 	} else
-		panic("vpci_init_iommu: getprop failed");
+		panic("vpci_init_iommu: getprop virtual-dma failed");
+
+	aprint_normal(" vdma %x length %x\n", iobase, iolen);
+	aprint_naive("\n");
 	
 	/* We have no STC.  */
 	is->is_sb[0] = NULL;
