@@ -1,4 +1,4 @@
-/*	$NetBSD: cpu.c,v 1.124 2015/03/22 19:37:54 palle Exp $ */
+/*	$NetBSD: cpu.c,v 1.127 2015/09/06 23:48:39 nakayama Exp $ */
 
 /*
  * Copyright (c) 1996
@@ -52,7 +52,7 @@
  */
 
 #include <sys/cdefs.h>
-__KERNEL_RCSID(0, "$NetBSD: cpu.c,v 1.124 2015/03/22 19:37:54 palle Exp $");
+__KERNEL_RCSID(0, "$NetBSD: cpu.c,v 1.127 2015/09/06 23:48:39 nakayama Exp $");
 
 #include "opt_multiprocessor.h"
 
@@ -72,10 +72,10 @@ __KERNEL_RCSID(0, "$NetBSD: cpu.c,v 1.124 2015/03/22 19:37:54 palle Exp $");
 #include <machine/pmap.h>
 #include <machine/sparc64.h>
 #include <machine/openfirm.h>
+#include <machine/hypervisor.h>
+#include <machine/mdesc.h>
 
 #include <sparc64/sparc64/cache.h>
-#include <sparc64/hypervisor.h>
-#include <sparc64/mdesc.h>
 
 #define SUN4V_MONDO_QUEUE_SIZE	32
 #define SUN4V_QUEUE_ENTRY_SIZE	64
@@ -112,14 +112,6 @@ static void cpu_reset_fpustate(void);
 
 volatile int sync_tick = 0;
 
-volatile uint64_t skod = 0;
-volatile uint64_t skod_o0 = 0;
-volatile uint64_t skod_o1 = 0;
-volatile uint64_t skod_o2 = 0;
-volatile uint64_t skod_o3 = 0;
-volatile uint64_t skod_o4 = 0;
-volatile uint64_t skod_o5 = 0;
-
 /* The CPU configuration driver. */
 void cpu_attach(device_t, device_t, void *);
 int cpu_match(device_t, cfdata_t, void *);
@@ -271,7 +263,7 @@ cpu_dcache_associativity(int node)
 		return prom_getpropint(node, "dcache-associativity", 1);
 }
 
-static int
+int
 cpu_ecache_size(int node)
 {
 	if (CPU_ISSUN4V)
@@ -298,7 +290,7 @@ cpu_ecache_nlines(int node)
 		return prom_getpropint(node, "ecache-nlines", 32768);
 }
 
-static int
+int
 cpu_ecache_associativity(int node)
 {
 	if (CPU_ISSUN4V) {
@@ -348,8 +340,6 @@ alloc_cpuinfo(u_int cpu_node)
 
 	memset((void *)va0, 0, sz);
 
-	printf("alloc_bootinfo() %d %p %p\n", cpuid, (void*)va0, (void*)pa0);
-	
 	/*
 	 * Initialize cpuinfo structure.
 	 *
@@ -680,8 +670,9 @@ cpu_boot_secondary_processors(void)
 		return;
 	}
 
-	/* FIXME */
-	/*return;*/
+	/* No MP for SUN4V yet */
+	if (CPU_ISSUN4V)
+		return;
 	
 	for (ci = cpus; ci != NULL; ci = ci->ci_next) {
 		if (ci->ci_cpuid == cpu_myid())
@@ -689,11 +680,8 @@ cpu_boot_secondary_processors(void)
 
 		cpu_pmap_prepare(ci, false);
 		cpu_args->cb_node = ci->ci_node;
-		cpu_args->cb_cpuinfo_va = (vaddr_t)ci;
-		cpu_args->cb_cpuinfo_pa = ci->ci_paddr;
+		cpu_args->cb_cpuinfo = ci->ci_paddr;
 		cpu_args->cb_cputyp = cputyp;
-		printf("%d %d %p %p\n", ci->ci_cpuid, cpu_args->cb_cputyp,
-		    (void*)cpu_args->cb_cpuinfo_va, (void*)cpu_args->cb_cpuinfo_pa);
 		membar_Sync();
 
 		/* Disable interrupts and start another CPU. */
@@ -707,17 +695,6 @@ cpu_boot_secondary_processors(void)
 			    (void *)cpu_spinup_trampoline, 0);
 
 		for (i = 0; i < 2000; i++) {
-#if 1			
-			printf("skod = %lx\n", skod);
-			printf("skod_o0 = %lx\n", skod_o0);
-			printf("skod_o1 = %lx\n", skod_o1);
-			printf("skod_o2 = %lx\n", skod_o2);
-			printf("skod_o3 = %lx\n", skod_o3);
-			printf("skod_o4 = %lx\n", skod_o4);
-			printf("skod_o5 = %lx\n", skod_o5);
-			if (skod_o5 == 25)
-				panic("EINVAL");
-#endif
 			membar_Sync();
 			if (CPUSET_HAS(cpus_active, ci->ci_index))
 				break;
@@ -744,7 +721,6 @@ cpu_boot_secondary_processors(void)
 void
 cpu_hatch(void)
 {
-	/* FIXME */
 	char *v = (char*)CPUINFO_VA;
 	int i;
 
@@ -771,6 +747,5 @@ cpu_hatch(void)
 		tickintr_establish(PIL_CLOCK, tickintr);
 	}
 	spl0();
-	printf("cpu_hatch() %d\n", cpu_myid());
 }
 #endif /* MULTIPROCESSOR */
